#!/usr/bin/env ruby
# -*- encoding: binary -*-
# Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
# License: GPLv3 or later (see COPYING for details)
require 'test/test_helper'
require 'io/wait'

class TestHTTPIdleExpire < Test::Unit::TestCase
  def setup
    @tmpdir = Dir.mktmpdir('cmogstored-http-test')
    @to_close = []
    @host = TEST_HOST

    srv = TCPServer.new(@host, 0)
    @http_port = srv.addr[1]
    srv.close

    srv = TCPServer.new(@host, 0)
    @mgmt_port = srv.addr[1]
    srv.close

    cmd = [ "cmogstored", "--docroot=#@tmpdir",
            "--httplisten=#@host:#@http_port",
            "--mgmtlisten=#@host:#@mgmt_port",
            "--maxconns=500" ]
    @err = Tempfile.new("stderr")
    nofile = 20

    # libkqueue uses many extra FDs per kqueue(), and we have two kqueue()
    cmd_path = %x{which cmogstored}.chomp
    kq = %x{ldd #{cmd_path}}.chomp.split(/\n/).grep(/libkqueue/)[0]
    nofile += 16 if kq

    @pid = fork {
      Process.setrlimit(Process::RLIMIT_NOFILE, nofile)
      # $stderr.reopen(@err)
      @err.close
      exec(*cmd)
    }
    @idle_timeout = 5
  end

  def test_idle_expire
    clients = [ get_client(300, @http_port) ]
    clients[0].write 'G'
    3.times do |i|
      c = get_client(1, @http_port)
      assert_kind_of TCPSocket, c
      clients << c
    end
    sleep(@idle_timeout + 1)
    new_clients = []
    4.times do |i|
      c = get_client(1, @http_port)
      assert_kind_of TCPSocket, c
      new_clients << c
    end
    clients.each do |c|
      assert_equal [[c],[],[]], IO.select([c], nil, nil, 1)
      assert_nil c.gets, "expect nil (EOF)"
      c.close
    end
    new_clients.each do |c|
      c.write("GET / HTTP/1.0\r\n\r\n")
      buf = c.read
      assert_match(%r{\AHTTP/1\.1 }, buf)
      assert_match(%r{\r\n\r\n\z}, buf)
      c.close
    end
  end

  def teardown
    Process.kill(:QUIT, @pid) rescue nil
    _, status = Process.waitpid2(@pid)
    @to_close.each { |io| io.close unless io.closed? }
    FileUtils.rm_rf(@tmpdir)
    @err.rewind
    $stderr.write(@err.read)
    assert status.success?, status.inspect
  end
end if RUBY_PLATFORM =~ /linux/
